package Plugins::Constellation::Plugin;

# HelloWorld tutorial plugin by Mitch Gerdisch
# Meant to be a way of figuring out and understanding how plugins need to be written for SC7.
# There are 4 basic components that need to be written for SC7 plugins:
#	Plugin.pm - This is the main processing bit of the plugin and not much different than pre-SC7 plugin code.
#				But, there are differences.
#	Settings.pm - This file contains perl code for processing the web page used to set the settings for the plugin.
#	strings.txt - Where all the strings are stored - including their various lanuage equivalents.
#	basic.html - This is a file under HTML/<language abbrev.>/plugins/<plugin name>/settings/. It contains 
#				 HTML-like stuff for displaying the plugin's web user interface.

# This code is derived from code with the following copyright message:
#
# SqueezeCenter Copyright 2001-2007 Logitech.
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2.

use strict;

use base qw(Slim::Plugin::Base);

use Plugins::Constellation::Settings;

use Slim::Music::Info;

use Slim::Utils::Misc;

use Slim::Utils::Network;

use Data::Dump;

use URI::Escape qw(uri_escape uri_escape_utf8);

use Encode qw(decode encode);

use Slim::Player::Player;

use Slim::Player::Client;

use Slim::Player::Sync;

use Scalar::Util qw(blessed);

use Slim::Control::Request;

use Slim::Utils::Log;

use Slim::Utils::Prefs;

use Slim::Utils::Strings qw(string);

use constant APP_URL => 'plugins/Constellation/html/index.html';

use constant GUI_SRC => '/plugins/Constellation/';

my $constellationipaddress;

my $constellationapi;

my $volume_test;

my $prefs = preferences('plugin.constellation');

our @browseMenuChoices;

my $log = Slim::Utils::Log->addLogCategory({
	'category'     => 'plugin.constellation',
#	'defaultLevel' => 'INFO',
	'defaultLevel' => 'DEBUG',
	'description'  => getDisplayName(),
});

sub getDisplayName {
	return 'PLUGIN_CONSTELLATION';
}

# I have my own debug routine so that I can add the *** stuff easily.
sub myDebug {
	my $msg = shift;
	my $lvl = shift;
	
	if ($lvl eq "")
	{
		$lvl = "debug";
	}

	
	$log->$lvl("*** Constellation *** $msg");
}

sub initPlugin {

	$constellationipaddress =	$prefs->get('address');
    
	$constellationapi  =	$prefs->get('user');
    
    my $class = shift;
    
    myDebug("Initializing");
        
    $class->SUPER::initPlugin();

    Plugins::Constellation::Settings->new;
    
    Slim::Web::Pages->addPageLinks("plugins", { 'PLUGIN_CONSTELLATION_GUI' => APP_URL });

    Slim::Control::Request::subscribe( \&commandCallbackClient, [['client']]);
    
    Slim::Control::Request::subscribe( \&commandCallbackVolume, [['mixer']]);
    
    Slim::Control::Request::subscribe( \&commandCallbackPower, [['power']]);
    
    Slim::Control::Request::subscribe( \&commandCallbackSync, [['sync']]);
    
    Slim::Control::Request::subscribe( \&commandCallbackShuffle, [['shuffle']]);
    
    Slim::Control::Request::subscribe( \&commandCallbackRepeat, [['repeat']]);
        
    Slim::Control::Request::subscribe( \&commandCallback, [['play', 'pause']]);   
      
}

sub shutdownPlugin {

    Slim::Control::Request::unsubscribe(\&commandCallbackClient);
    
    Slim::Control::Request::unsubscribe(\&commandCallbackVolume);
    
    Slim::Control::Request::unsubscribe(\&commandCallbackPower);
    
    Slim::Control::Request::unsubscribe(\&commandCallbackSync);
    
    Slim::Control::Request::unsubscribe(\&commandCallbackShuffle);
    
    Slim::Control::Request::unsubscribe(\&commandCallbackRepeat);
    
    Slim::Control::Request::unsubscribe(\&commandCallback);

}



sub commandCallbackClient {

    my $request = shift;
    
    my $cmd = $request->getResults();
    
    my $client = $request->client();
    
    my @players = Slim::Player::Client::clients();
    
    my $test = '[';
    
    foreach my $client (@players) {
        
        my $client_connect = $client->connected();
        
        if($client_connect ==1){
           
            my $client_name = $client->name();

            $client_name = encode('UTF-8', $client_name);   
                
            my $client_mac = ref($client) ? $client->macaddress() : $client; 
            
            $test .= "{name:\"$client_name\",mac:\"$client_mac\"},";  
        
        }
    }
    
    $test .= ']';
    
    $log->debug($test);
    
    if( !defined( $client)) {
    
        return;
        
    }
    my $mac = ref($client) ? $client->macaddress() : $client;

    if( $request->isCommand([['client'], ['new']]) || $request->isCommand([['client'], ['reconnect']])) {
    
       &handlePlayTrack($client);
       
       &sendToConstellation ( "Players", $test);

    }elsif( $request->isCommand([['client'], ['disconnect']])) {
    
       	&sendToConstellation ( "Players", $test);
      
    	my $player_name = $client->name();

        my $url = "http://$constellationipaddress/rest/constellation/PurgeStateObjects?SentinelName=Squeezebox&PackageName=Info&AccessKey=$constellationapi&name=$player_name";
     	
        my $http = Slim::Networking::SimpleAsyncHTTP->new(\&exampleCallback,\&exampleErrorCallback,{'mydata'  => 'foo',});

        $log->debug($test);
          
        $http->get($url, 'Content-Type' => 'application/json');
    
    }
    
}

sub commandCallbackVolume {

    my $request = shift;

    my $client = $request->client();
    
    if( !defined( $client)) {
    
        return;
    }
    
    $log->debug("Callback on volume");
    
    my $iVolume = $client->volume();
        
    # $log->debug("Request result: " .Data::Dump::dump($request));
    
    $volume_test = $iVolume;
    
    # foreach my $artist ($request){
        
    #     $log->debug("Artist");
          
    # }
    
    &handlePlayTrack($client);
        
}

sub commandCallbackSync {

    my $request = shift;

    my $client = $request->client();
          
    if( !defined( $client)) {
    
        return;
        
    }
    
    $log->debug("Callback on sync");
    
    &handlePlayTrack($client);

}

sub commandCallbackShuffle {

    my $request = shift;

    my $client = $request->client();
     
    if( !defined( $client)) {
    
        return;
        
    }
      
    $log->debug("Callback on shuffle"); 
    
    &handlePlayTrack($client);
 
}
 
sub commandCallbackRepeat {

    my $request = shift;

    my $client = $request->client();
    
    if( !defined( $client)) {
    
        return;
        
    }
      
    $log->debug("Callback on repeat");  
    
    &handlePlayTrack($client);
    
}
 
sub commandCallbackPower {

    my $request = shift;
    
    my $client = $request->client();
    
    if( !defined( $client)) {
    
        return;
        
    }
        
    my $iSyncedPlayer = $client->isSynced() ;
    
    my $macmaster = $client->master->macaddress() ;
    
    my $masterpower = $client->master->power() ;
    
    my @SyncedSlaves ;

    my $iPower = $client->power();
    
    my $iPaused = $client->isPaused();
        
    my $iStopped = $client->isStopped();
       
    my $iVolume = $client->volume();
        
    my $mac = ref($client) ? $client->macaddress() : $client;
    
    if($iPower ==  1) {
   
        if($iPaused ne  1) {
        
            if ($iStopped == 1){
            
                &handlePlayStop ($client);
                
            }
            
        }else{
        
            &handlePlayPause ($client);
            
        }
        
        &sendToConstellation ( $mac, "volume_status=$iVolume");
        
    }else{
    
        if ($iSyncedPlayer ==1 ){
    
            @SyncedSlaves = Slim::Player::Sync::slaves($client);                                      
            
            if ($masterpower ==0 ){
            
                $mac = $macmaster;
            
            }
            
            foreach my $slaveclient (@SyncedSlaves) {

                    my $macslave = ref($slaveclient) ? $slaveclient->macaddress() : $slaveclient;   

                    my $iPower = $slaveclient->power();

                    if($iPower ==  0){

                        $mac .= '/'.$macslave; 

                    }
            }
    
        &sendToConstellation ( $mac, "status=Eteint");
        
       }else{
       
        &sendToConstellation ( $mac, "status=Eteint");
       
       }
       
    }
    
}

sub commandCallback {

    my $request = shift;

    my $client = $request->client();

        if( !defined( $client)) {
    
        return;
        
    }

    if(($request->isCommand([['playlist'], ['addtracks']])) || ($request->isCommand([['playlist'], ['open']]))) {
    }else{
    	#$log->debug("Request result: " .Data::Dump::dump($request));  
    	&handlePlayTrack($client);
    }
           
}

sub handlePlayTrack {

    my $client = shift;
    
    my $iSyncedPlayer = $client->isSynced() ;
    
    my $iPower = $client->power();
    
    my $iMaster = Slim::Player::Sync::isMaster($client);
    
    my @iSyncedMaster = $client->master() ;
    
    my $iSyncedGroup = "";
    
    my @SyncedSlaves ;
    
    my $sTitle = $client->playingSong();
    
    my $sName =  'Vide';    
    
    my $artist   = 'Vide';
    
    my $artist_id   = 'null';
    
    my $album    = 'Vide';
    
    my $album_id    = 'null';
    
    my $id    = 'null';
    
    my $cover = 'null';
    
    my $duration =  0;
    
    my $played =  0;
    
    my $iVolume = $client->volume();
      
    my $player_name = $client->name();
    
    my $player_model = $client->modelName();
    
    my $player_ip = $client->ip();
    
    my $status = ' ';
    
    my $sync_status = 0;
    
    my $repeat_status = Slim::Player::Playlist::repeat($client);
    
    my $shuffle_status = Slim::Player::Playlist::shuffle($client);
    
    my $mac = ref($client) ? $client->macaddress() : $client;

    if(defined($sTitle)) {
    
        eval {$sName =  $sTitle->track()->title  || ''; }; warn $@ if $@;
        
        eval {$id =  $sTitle->track()->id  || '';   }; warn $@ if $@;
        
        eval { $artist   = $sTitle->track()->artistName || '';}; warn $@ if $@;
        
        eval { $artist_id   = $sTitle->track()->artistid || '';}; warn $@ if $@;
        
        eval { $album    = $sTitle->track()->album  ? $sTitle->track()->album->name  : '';}; warn $@ if $@;
        
        eval { $album_id    = $sTitle->track()->albumid || '';}; warn $@ if $@;      
        
#        eval { $cover = $sTitle->track()->id || '';}; warn $@ if $@;
        
        eval { $duration = $sTitle->track()->secs;}; warn $@ if $@;
    
#        $cover = 'music/'.$cover.'/cover.jpg';
        
        $cover = 'music/current/cover.jpg?player='.$mac.'&album_id='.$album_id;
        
        # $cover = uri_escape_utf8($cover);
        
        #  $artist = uri_escape_utf8($artist);
        $artist     = encode('UTF-8', $artist);
        
        #  $album = uri_escape_utf8($album);
        $album     = encode('UTF-8', $album);

        #  $sName = uri_escape_utf8($sName);
        $sName     = encode('UTF-8', $sName);
        
#       $log->debug("Request result: " .$sName.' album= '.$album);
    
        my $remoteMeta;
        
        my $handler;
        
        $handler = Slim::Player::ProtocolHandlers->handlerForURL($sTitle->track()->url);
            
        if ( $handler && $handler->can('getMetadataFor') ) {
        
            $remoteMeta = $handler->getMetadataFor($client,$sTitle->track()->url );
            
            $cover = 'music/current/cover.jpg?player='.$mac.'&'.rand();
            
            $album = $sName;
            
            # $log->debug('Request result: album= '.$album);
             
#           $cover = uri_escape_utf8($remoteMeta->{cover});

            if(defined( $remoteMeta->{album})) {
            
                # $album = uri_escape_utf8($remoteMeta->{album});
                $album = encode('UTF-8', $remoteMeta->{album});
            
            }
            
            # $artist = uri_escape_utf8($remoteMeta->{artist});
            
            # $sName = uri_escape_utf8($remoteMeta->{title});
            
            $artist = encode('UTF-8', $remoteMeta->{artist});
            
            $sName = encode('UTF-8', $remoteMeta->{title});
            
            $artist_id = "null";
            
            $album_id = "null";
            
            # $log->debug("Request result: " .Data::Dump::dump($remoteMeta));
            
        }
        
        $played = Slim::Player::Source::songTime($client);
        
    }
    
    

    #   $duration = floor($duration);
    
    my $remaining = $duration - $played;
    
    my $iPaused = $client->isPaused();
        
    my $iStopped = $client->isStopped();
    
    if($iPower ==  1) {
            
        if($iPaused ne  1) {
            
            if ($iStopped == 1){
            
                $status = "Stop";
                
            }else{
                
               $status = "Lecture"; 
               $volume_test = $iVolume;
            }
       
        }else{
            
            $status = "Pause";
            
        } 
            
    }else{
        
        $status = "Off";
    }
        
    if ($iSyncedPlayer ==1 ){
        
        $sync_status = 1;
    
        @SyncedSlaves = Slim::Player::Sync::slaves($client);
        
        my $sum;
        
        my $slave_json;
        
        foreach my $slaveclient (@SyncedSlaves) {
            
            my $macslave = ref($slaveclient) ? $slaveclient->macaddress() : $slaveclient;
           
            my $slave_name = $slaveclient->name();
    
            $slave_json .="{name:\"$slave_name\",mac:\"$macslave\"},"
                
       }
       
      
        foreach my $slaveclient (@SyncedSlaves) {
            
            my $macslave = ref($slaveclient) ? $slaveclient->macaddress() : $slaveclient;
           
            my $slave_name = $slaveclient->name();
    
            my $slave_model = $slaveclient->modelName();
    
            my $slave_ip = $slaveclient->ip();

            if ($sum eq ""){
                
                $sum .= $macslave;
            
            }else{
                
                $sum .= '/'.$macslave; 
                        
            }
            
            &sendToConstellation ( $slave_name, "{name:\"$slave_name\",mac:\"$macslave\",ip:\"$slave_ip\",model:\"$slave_model\",repeat:$repeat_status,shuffle:$shuffle_status,data:{title:\"$sName\",title_id:$id,artist:\"$artist\",artist_id:$artist_id,album:\"$album\",album_id:$album_id,cover:\"$cover\\\",duration:$duration,remaining:$remaining,statut:\"$status\",volume:$volume_test,sync:{is_master:false,master:{mac:\"$mac\",name:\"$player_name\"},slave:[$slave_json]}}}");
    
       }

       foreach my $masterclient (@iSyncedMaster) {
       
            my $macmaster = ref($masterclient) ? $masterclient->macaddress() : $masterclient;   

            $sum .= '/'.$macmaster;

        } 

        &sendToConstellation ( $player_name, "{name:\"$player_name\",mac:\"$mac\",ip:\"$player_ip\",model:\"$player_model\",repeat:$repeat_status,shuffle:$shuffle_status,data:{title:\"$sName\",title_id:$id,artist:\"$artist\",artist_id:$artist_id,album:\"$album\",album_id:$album_id,cover:\"$cover\",duration:$duration,remaining:$remaining,statut:\"$status\",volume:$volume_test,sync:{is_master:true,master:null,slave:[$slave_json]}}}");

    }else{
    
        &sendToConstellation ( $player_name, "{name:\"$player_name\",mac:\"$mac\",ip:\"$player_ip\",model:\"$player_model\",repeat:$repeat_status,shuffle:$shuffle_status,data:{title:\"$sName\",title_id:$id,artist:\"$artist\",artist_id:$artist_id,album:\"$album\",album_id:$album_id,cover:\"$cover\",duration:$duration,remaining:$remaining,statut:\"$status\",volume:$volume_test,sync:null}}");
    
    }
        
        
}


sub sendToConstellation {  

    my $player_name = shift;
    
    $player_name = encode('UTF-8', $player_name);

    my $action = shift;

    my $url = "http://$constellationipaddress/rest/constellation/PushStateObject?SentinelName=Squeezebox&PackageName=Info&AccessKey=$constellationapi";
 
    my $http = Slim::Networking::SimpleAsyncHTTP->new(\&exampleCallback,\&exampleErrorCallback,{'mydata'  => 'foo',});

    my $test = "{name:\"$player_name\",Value:$action";

    $log->debug($test);
      
    $http->post($url, 'Content-Type' => 'application/json', $test);

}

sub exampleCallback {
    my $http = shift;

    my $content = $http->content();

    my $data = $http->params('mydata');

    print("Got the content and my data.\n");
}

sub exampleErrorCallback {

}
1;